@@ -159,30 +159,26 @@ void CueControl::createControls() {
     }
 }
 
-void CueControl::attachCue(CuePointer pCue, int hotCue) {
-    HotcueControl* pControl = m_hotcueControls.value(hotCue, NULL);
-    if (pControl == NULL) {
+void CueControl::attachCue(CuePointer pCue, HotcueControl* pControl) {
+    VERIFY_OR_DEBUG_ASSERT(pControl) {
         return;
     }
-    if (pControl->getCue() != NULL) {
-        detachCue(pControl->getHotcueNumber());
-    }
+    detachCue(pControl);
     connect(pCue.get(), SIGNAL(updated()),
             this, SLOT(cueUpdated()),
             Qt::DirectConnection);
 
     pControl->setCue(pCue);
-
 }
 
-void CueControl::detachCue(int hotCue) {
-    HotcueControl* pControl = m_hotcueControls.value(hotCue, NULL);
-    if (pControl == NULL) {
+void CueControl::detachCue(HotcueControl* pControl) {
+    VERIFY_OR_DEBUG_ASSERT(pControl) {
         return;
     }
     CuePointer pCue(pControl->getCue());
-    if (!pCue)
+    if (!pCue) {
         return;
+    }
     disconnect(pCue.get(), 0, this, 0);
     pControl->resetCue();
 }
@@ -191,8 +187,8 @@ void CueControl::trackLoaded(TrackPointer pNewTrack) {
     QMutexLocker lock(&m_mutex);
     if (m_pLoadedTrack) {
         disconnect(m_pLoadedTrack.get(), 0, this, 0);
-        for (int i = 0; i < m_iNumHotCues; ++i) {
-            detachCue(i);
+        for (const auto& pControl: m_hotcueControls) {
+            detachCue(pControl);
         }
 
         m_pCueIndicator->setBlinkValue(ControlIndicator::OFF);
@@ -211,33 +207,37 @@ void CueControl::trackLoaded(TrackPointer pNewTrack) {
 
     CuePointer pLoadCue;
     for (const CuePointer& pCue: m_pLoadedTrack->getCuePoints()) {
-        if (pCue->getType() == Cue::CUE) {
-            continue; // skip
-        }
         if (pCue->getType() == Cue::LOAD) {
             DEBUG_ASSERT(!pLoadCue);
             pLoadCue = pCue;
         }
-        int hotcue = pCue->getHotCue();
-        if (hotcue != -1) {
-            attachCue(pCue, hotcue);
-        }
     }
-    double cuePoint;
-    if (pLoadCue) {
-        cuePoint = pLoadCue->getPosition();
-    } else {
-        // If no load cue point is stored, read from track
-        cuePoint = m_pLoadedTrack->getCuePoint();
-    }
-    m_pCuePoint->set(cuePoint);
 
     // Need to unlock before emitting any signals to prevent deadlock.
     lock.unlock();
-
-    // Use pNewTrack here, because m_pLoadedTrack might have been reset
+    // Use pNewTrack from now, because m_pLoadedTrack might have been reset
     // immediately after leaving the locking scope!
-    pNewTrack->setCuePoint(cuePoint);
+
+
+    // Because of legacy, we store the (load) cue point twice and need to
+    // sync both values.
+    // The Cue::LOAD from getCuePoints() has the priority
+    double cuePoint;
+    if (pLoadCue) {
+        cuePoint = pLoadCue->getPosition();
+        // adjust the track cue accordingly
+        pNewTrack->setCuePoint(cuePoint);
+    } else {
+        // If no load cue point is stored, read from track
+        // Note: This is 0:00 for new tracks
+        cuePoint = pNewTrack->getCuePoint();
+        // Than add the load cue to the list of cue
+        CuePointer pCue(pNewTrack->createAndAddCue());
+        pCue->setPosition(cuePoint);
+        pCue->setHotCue(-1);
+        pCue->setType(Cue::LOAD);
+    }
+    m_pCuePoint->set(cuePoint);
 
     // If cue recall is ON in the prefs, then we're supposed to seek to the cue
     // point on song load. Note that [Controls],cueRecall == 0 corresponds to "ON", not OFF.
@@ -268,32 +268,33 @@ void CueControl::trackCuesUpdated() {
     if (!m_pLoadedTrack)
         return;
 
+    bool loadCueFound = false;
     const QList<CuePointer> cuePoints(m_pLoadedTrack->getCuePoints());
     QListIterator<CuePointer> it(cuePoints);
     while (it.hasNext()) {
         CuePointer pCue(it.next());
 
-        if (pCue->getType() != Cue::CUE && pCue->getType() != Cue::LOAD)
+        if (!loadCueFound && pCue->getType() == Cue::LOAD) {
+            loadCueFound = true;
+            m_pCuePoint->set(pCue->getPosition());
             continue;
+        }
+
+        if (pCue->getType() != Cue::CUE) {
+            continue;
+        }
 
         int hotcue = pCue->getHotCue();
-        if (hotcue != -1) {
-            HotcueControl* pControl = m_hotcueControls.value(hotcue, NULL);
-
-            // Cue's hotcue doesn't have a hotcue control.
-            if (pControl == NULL) {
-                continue;
-            }
+        HotcueControl* pControl = m_hotcueControls.value(hotcue);
+        if (pControl) {
+            // Note: The Cue's hotcue doesn't have a hotcue control.
 
             CuePointer pOldCue(pControl->getCue());
 
             // If the old hotcue is different than this one.
             if (pOldCue != pCue) {
-                // If the old hotcue exists, detach it
-                if (pOldCue) {
-                    detachCue(hotcue);
-                }
-                attachCue(pCue, hotcue);
+                // old cue is detached if required
+                attachCue(pCue, pControl);
             } else {
                 // If the old hotcue is the same, then we only need to update
                 pControl->setPosition(pCue->getPosition());
@@ -303,10 +304,15 @@ void CueControl::trackCuesUpdated() {
         }
     }
 
+    if (!loadCueFound) {
+        m_pCuePoint->set(-1);
+    }
+
     // Detach all hotcues that are no longer present
-    for (int i = 0; i < m_iNumHotCues; ++i) {
-        if (!active_hotcues.contains(i)) {
-            detachCue(i);
+    for (int hotCue = 0; hotCue < m_iNumHotCues; ++hotCue) {
+        if (!active_hotcues.contains(hotCue)) {
+            HotcueControl* pControl = m_hotcueControls.at(hotCue);
+            detachCue(pControl);
         }
     }
 }
@@ -338,7 +344,7 @@ void CueControl::hotcueSet(HotcueControl* pControl, double v) {
     pCue->setLabel("");
     pCue->setType(Cue::CUE);
     // TODO(XXX) deal with spurious signals
-    attachCue(pCue, hotcue);
+    attachCue(pCue, pControl);
 
     // If quantize is enabled and we are not playing, jump to the cue point
     // since it's not necessarily where we currently are. TODO(XXX) is this
@@ -456,11 +462,12 @@ void CueControl::hotcueActivate(HotcueControl* pControl, double v) {
             }
         }
     } else {
+        // The cue is non-existent ...
         if (v) {
-            // just in case
+            // set it to the current position
             hotcueSet(pControl, v);
         } else if (m_iCurrentlyPreviewingHotcues) {
-            // The cue is non-existent, yet we got a release for it and are
+            // yet we got a release for it and are
             // currently previewing a hotcue. This is indicative of a corner
             // case where the cue was detached while we were pressing it. Let
             // hotcueActivatePreview handle it.
@@ -520,7 +527,10 @@ void CueControl::hotcueClear(HotcueControl* pControl, double v) {
     }
 
     CuePointer pCue(pControl->getCue());
-    detachCue(pControl->getHotcueNumber());
+    if (!pCue) {
+        return;
+    }
+    detachCue(pControl);
     m_pLoadedTrack->removeCue(pCue);
 }
 
@@ -533,8 +543,7 @@ void CueControl::hotcuePositionChanged(HotcueControl* pControl, double newPositi
     if (pCue) {
         // Setting the position to -1 is the same as calling hotcue_x_clear
         if (newPosition == -1) {
-            pCue->setHotCue(-1);
-            detachCue(pControl->getHotcueNumber());
+            detachCue(pControl);
         } else if (newPosition > 0 && newPosition < m_pTrackSamples->get()) {
             pCue->setPosition(newPosition);
         }
