@@ -11,7 +11,6 @@
 
 #include "preferences/usersettings.h"
 #include "skin/skincontext.h"
-#include "util/class.h"
 #include "waveform/renderers/waveformmarkrange.h"
 #include "waveform/renderers/waveformrendererabstract.h"
 
@@ -21,17 +20,15 @@ class ControlObject;
 class WaveformRenderMarkRange : public WaveformRendererAbstract {
   public:
     explicit WaveformRenderMarkRange(WaveformWidgetRenderer* waveformWidgetRenderer);
-    virtual ~WaveformRenderMarkRange();
+    ~WaveformRenderMarkRange() override = default;
 
-    virtual void setup(const QDomNode& node, const SkinContext& context);
-    virtual void draw(QPainter* painter, QPaintEvent* event);
+    void setup(const QDomNode& node, const SkinContext& context) override;
+    void draw(QPainter* painter, QPaintEvent* event) override;
 
   private:
     void generateImages();
 
     std::vector<WaveformMarkRange> m_markRanges;
-
-    DISALLOW_COPY_AND_ASSIGN(WaveformRenderMarkRange);
 };
 
 #endif
