@@ -32,7 +32,7 @@ class WaveformWidgetAbstract : public WaveformWidgetRenderer {
 
     virtual void preRender(VSyncThread* vsyncThread);
     virtual mixxx::Duration render();
-    virtual void resize(int width, int height, float devicePixelRatio) override;
+    virtual void resize(int width, int height);
 
   protected:
     QWidget* m_widget;
